<?php

namespace Drupal\webform_redis_limit_handler\Plugin\WebformHandler;

use Drupal\Core\Form\FormStateInterface;
use Drupal\webform\Plugin\WebformHandlerBase;
use Drupal\webform\WebformInterface;
use Drupal\webform\WebformSubmissionInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Form submission handler.
 *
 * Limit a webform submission form.
 *
 * @WebformHandler(
 *   id = "redis_limit_submission",
 *   label = @Translation("Redis limit submission"),
 *   category = @Translation("Webform Handler"),
 *   description = @Translation("Check the limit submission"),
 *   cardinality = \Drupal\webform\Plugin\WebformHandlerInterface::CARDINALITY_SINGLE,
 *   results = \Drupal\webform\Plugin\WebformHandlerInterface::RESULTS_PROCESSED,
 * )
 */
class RedisLimitSubmissionHandler extends WebformHandlerBase {

  /**
   * The Redis connection.
   *
   * @var \Predis\Client
   */
  protected $client;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    $instance = parent::create($container, $configuration, $plugin_id, $plugin_definition);
    $instance->client = $container->get('redis.factory')->getClient();
    return $instance;
  }

  /**
   * Return the key store total submit in redis.
   */
  private function getTotalRedisKey() {
    return 'webform_total_key_' . $this->webform->id();
  }

  /**
   * Init redis value.
   */
  private function initRedis($total_redis_key): void {
    if (!$this->client->get($total_redis_key)) {
      $total = $this->entityTypeManager
        ->getStorage('webform_submission')
        ->getQuery()
        ->condition('webform_id', $this->webform->id())
        ->accessCheck(FALSE)
        ->count()
        ->execute();
      $this->client->setnx($total_redis_key, $total);
    }
  }

  /**
   * {@inheritdoc}
   */
  public function alterForm(array &$form, FormStateInterface $form_state, WebformSubmissionInterface $webform_submission) {
    if ($form_state->get('current_page') !== WebformInterface::PAGE_CONFIRMATION) {
      $total = $this->client->get($this->getTotalRedisKey());
      if (is_numeric($this->getSetting('limit'))
        && $total >= $this->getSetting('limit')
      ) {
        $message = $this->getSetting('warning_message') ?? $this->t('Submission limit has been reached.!');

        $this->messenger()->addWarning($message);
      }
      return parent::alterForm($form, $form_state, $webform_submission);
    }
    else {
      $form['#attributes']['class'][] = 'webform-submission--confirmation';
    }
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $form = parent::buildConfigurationForm($form, $form_state);
    $form['total_submission'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Total submissions'),
    ];

    $form['total_submission']['limit'] = [
      '#type' => 'number',
      '#title' => $this->t('Total Limit'),
      '#return_value' => TRUE,
      '#default_value' => $this->configuration['limit'],

    ];

    $total_redis_key = $this->getTotalRedisKey();
    $this->initRedis($total_redis_key);

    $form['total_submission']['redis_counter'] = [
      '#type' => 'number',
      '#title' => $this->t('Current Redis Counter'),
      '#description' => $this->t('If the input value is less than the total submissions, it will be automatically saved as the total submissions.'),
      '#return_value' => TRUE,
      '#default_value' => $this->client->get($total_redis_key),
    ];

    $form['total_submission']['warning_message'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Warning message'),
      '#return_value' => TRUE,
      '#default_value' => $this->configuration['warning_message'],
    ];

    return $this->setSettingsParents($form);
  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state) {
    $total_redis_key = $this->getTotalRedisKey();
    parent::submitConfigurationForm($form, $form_state);
    $this->configuration['limit'] = $form_state->getValue(['total_submission', 'limit']);
    $this->configuration['warning_message'] = $form_state->getValue(['total_submission', 'warning_message']);

    $redis_counter = $form_state->getValue(['total_submission', 'redis_counter']);
    if (is_numeric($redis_counter)) {
      $total = $this->submissionStorage->getTotal();
      $redis_counter = $redis_counter >= $total ? $redis_counter : $total;
      $dx = $redis_counter - $this->client->get($total_redis_key);

      $this->client->incrby($total_redis_key, $dx);
    }
    else {
      $this->client->del($total_redis_key);
    }
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(&$form, FormStateInterface $form_state, WebformSubmissionInterface $webform_submission) {
    $total_redis_key = $this->getTotalRedisKey();

    if (!$form_state->hasAnyErrors()
      && is_array($form_state->getTriggeringElement())
      && $form_state->getTriggeringElement()['#name'] === 'op'
      && is_numeric($this->getSetting('limit'))
    ) {
      $this->initRedis($total_redis_key);

      $limit = $this->getSetting('limit');
      $total = $this->client->incrby($total_redis_key, 1);
      if ($total > $limit) {
        $form_state->setErrorByName('', $this->getSetting('warning_message'));
        $dx = $limit - $this->client->get($total_redis_key);
        $this->client->incrby($total_redis_key, $dx);
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public function getSummary() {
    return [];
  }

}
